#[derive(Debug, Clone)]
pub struct Position {
    pub index: usize,
    pub line: i32,
    pub col: i32,
}

impl Position {
    pub fn new(index: usize, line: i32, col: i32) -> Self {
        Self { index, line, col }
    }
}

#[derive(Debug, PartialEq)]
pub enum TokenType {
    InvalidChar,
    MalformedString,
    MalformedComment,

    Id,
    Int,
    Decimal,
    String,

    False,
    True,
    Let,
    Mut,
    If,
    Else,
    While,
    For,
    In,
    Break,
    Continue,
    Fn,
    Return,
    End,
    Not,
    And,
    Or,
    Underscore,

    Plus,
    Minus,
    Asterisk,
    Slash,
    Percent,
    DoubleAsterisk,
    Equal,
    Exclamation,
    LessThan,
    GreaterThan,

    PlusEqual,
    MinusEqual,
    AsteriskEqual,
    SlashEqual,
    PercentEqual,
    DoubleAsteriskEqual,
    EqualEqual,
    ExclamationEqual,
    LessThanEqual,
    GreaterThanEqual,

    LParen,
    RParen,
    LBrace,
    RBrace,
    LBracket,
    RBracket,
    Dot,
    Comma,
    Colon,
    Semicolon,
    Ampersand,
    DotDot,
    DotDotEqual,
    DotDotDot,
    EqualLessThan,
}

#[derive(Debug)]
pub struct Token {
    pub token_type: TokenType,
    pub pos: Position,
    pub length: usize,
}

pub trait PositionKnowing {
    fn pos(&self) -> Position;
}
